@@ -1,31 +1,10 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* libvisio
- * Version: MPL 1.1 / GPLv2+ / LGPLv2+
+/*
+ * This file is part of the libvisio project.
  *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License or as specified alternatively below. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * Major Contributor(s):
- * Copyright (C) 2011 Fridrich Strba <fridrich.strba@bluewin.ch>
- * Copyright (C) 2011 Eilidh McAdam <tibbylickle@gmail.com>
- *
- *
- * All Rights Reserved.
- *
- * For minor contributions see the git repository.
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPLv2+"), or
- * the GNU Lesser General Public License Version 2 or later (the "LGPLv2+"),
- * in which case the provisions of the GPLv2+ or the LGPLv2+ are applicable
- * instead of those above.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
 #ifndef VSDCONTENTCOLLECTOR_H
@@ -38,7 +17,6 @@
 #include <map>
 #include <list>
 #include <vector>
-#include <libwpg/libwpg.h>
 #include "libvisio_utils.h"
 #include "VSDCollector.h"
 #include "VSDParser.h"
@@ -53,7 +31,7 @@ class VSDContentCollector : public VSDCollector
 {
 public:
   VSDContentCollector(
-    libwpg::WPGPaintInterface *painter,
+    librevenge::RVNGDrawingInterface *painter,
     std::vector<std::map<unsigned, XForm> > &groupXFormsSequence,
     std::vector<std::map<unsigned, unsigned> > &groupMembershipsSequence,
     std::vector<std::list<unsigned> > &documentPageShapeOrders,
@@ -65,13 +43,13 @@ public:
   };
 
   void collectEllipticalArcTo(unsigned id, unsigned level, double x3, double y3, double x2, double y2, double angle, double ecc);
-  void collectForeignData(unsigned level, const WPXBinaryData &binaryData);
+  void collectForeignData(unsigned level, const librevenge::RVNGBinaryData &binaryData);
   void collectOLEList(unsigned id, unsigned level);
-  void collectOLEData(unsigned id, unsigned level, const WPXBinaryData &oleData);
+  void collectOLEData(unsigned id, unsigned level, const librevenge::RVNGBinaryData &oleData);
   void collectEllipse(unsigned id, unsigned level, double cx, double cy, double xleft, double yleft, double xtop, double ytop);
   void collectLine(unsigned level, const boost::optional<double> &strokeWidth, const boost::optional<Colour> &c, const boost::optional<unsigned char> &linePattern,
                    const boost::optional<unsigned char> &startMarker, const boost::optional<unsigned char> &endMarker,
-                   const boost::optional<unsigned char> &lineCap);
+                   const boost::optional<unsigned char> &lineCap, const boost::optional<double> &rounding);
   void collectFillAndShadow(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
                             const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
                             const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
@@ -80,6 +58,8 @@ public:
                             const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
                             const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
                             const boost::optional<Colour> &shfgc);
+  void collectThemeReference(unsigned level, const boost::optional<long> &lineColour, const boost::optional<long> &fillColour,
+                             const boost::optional<long> &shadowColour, const boost::optional<long> &fontColour);
   void collectGeometry(unsigned id, unsigned level, bool noFill, bool noLine, bool noShow);
   void collectMoveTo(unsigned id, unsigned level, double x, double y);
   void collectLineTo(unsigned id, unsigned level, double x, double y);
@@ -114,7 +94,7 @@ public:
 
   void collectUnhandledChunk(unsigned id, unsigned level);
 
-  void collectText(unsigned level, const WPXBinaryData &textStream, TextFormat format);
+  void collectText(unsigned level, const librevenge::RVNGBinaryData &textStream, TextFormat format);
   void collectCharIX(unsigned id, unsigned level, unsigned charCount, const boost::optional<VSDName> &font,
                      const boost::optional<Colour> &fontColour, const boost::optional<double> &fontSize, const boost::optional<bool> &bold,
                      const boost::optional<bool> &italic, const boost::optional<bool> &underline, const boost::optional<bool> &doubleunderline,
@@ -129,26 +109,33 @@ public:
   void collectParaIX(unsigned id, unsigned level, unsigned charCount, const boost::optional<double> &indFirst,
                      const boost::optional<double> &indLeft, const boost::optional<double> &indRight, const boost::optional<double> &spLine,
                      const boost::optional<double> &spBefore, const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
+                     const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr, const boost::optional<VSDName> &bulletFont,
+                     const boost::optional<double> &bulletFontSize, const boost::optional<double> &textPosAfterBullet,
                      const boost::optional<unsigned> &flags);
   void collectDefaultParaStyle(unsigned charCount, const boost::optional<double> &indFirst, const boost::optional<double> &indLeft,
                                const boost::optional<double> &indRight, const boost::optional<double> &spLine, const boost::optional<double> &spBefore,
-                               const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align, const boost::optional<unsigned> &flags);
+                               const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
+                               const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr,
+                               const boost::optional<VSDName> &bulletFont, const boost::optional<double> &bulletFontSize,
+                               const boost::optional<double> &textPosAfterBullet, const boost::optional<unsigned> &flags);
   void collectTextBlock(unsigned level, const boost::optional<double> &leftMargin, const boost::optional<double> &rightMargin,
                         const boost::optional<double> &topMargin, const boost::optional<double> &bottomMargin,
                         const boost::optional<unsigned char> &verticalAlign, const boost::optional<bool> &isBgFilled,
                         const boost::optional<Colour> &bgColour, const boost::optional<double> &defaultTabStop,
                         const boost::optional<unsigned char> &textDirection);
   void collectNameList(unsigned id, unsigned level);
-  void collectName(unsigned id, unsigned level,  const WPXBinaryData &name, TextFormat format);
+  void collectName(unsigned id, unsigned level,  const librevenge::RVNGBinaryData &name, TextFormat format);
   void collectPageSheet(unsigned id, unsigned level);
   void collectMisc(unsigned level, const VSDMisc &misc);
-
+  void collectLayer(unsigned id, unsigned level, const VSDLayer &layer);
+  void collectLayerMem(unsigned level, const VSDName &layerMem);
+  void collectTabsDataList(unsigned level, const std::map<unsigned, VSDTabSet> &tabSets);
 
   // Style collectors
   void collectStyleSheet(unsigned id, unsigned level, unsigned parentLineStyle, unsigned parentFillStyle, unsigned parentTextStyle);
   void collectLineStyle(unsigned level, const boost::optional<double> &strokeWidth, const boost::optional<Colour> &c, const boost::optional<unsigned char> &linePattern,
                         const boost::optional<unsigned char> &startMarker, const boost::optional<unsigned char> &endMarker,
-                        const boost::optional<unsigned char> &lineCap);
+                        const boost::optional<unsigned char> &lineCap, const boost::optional<double> &rounding);
   void collectFillStyle(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
                         const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
                         const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
@@ -166,12 +153,19 @@ public:
   void collectParaIXStyle(unsigned id, unsigned level, unsigned charCount, const boost::optional<double> &indFirst,
                           const boost::optional<double> &indLeft, const boost::optional<double> &indRight, const boost::optional<double> &spLine,
                           const boost::optional<double> &spBefore, const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
-                          const boost::optional<unsigned> &flags);
+                          const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr,
+                          const boost::optional<VSDName> &bulletFont, const boost::optional<double> &bulletFontSize,
+                          const boost::optional<double> &textPosAfterBullet, const boost::optional<unsigned> &flags);
   void collectTextBlockStyle(unsigned level, const boost::optional<double> &leftMargin, const boost::optional<double> &rightMargin,
                              const boost::optional<double> &topMargin, const boost::optional<double> &bottomMargin,
                              const boost::optional<unsigned char> &verticalAlign, const boost::optional<bool> &isBgFilled,
                              const boost::optional<Colour> &bgColour, const boost::optional<double> &defaultTabStop,
                              const boost::optional<unsigned char> &textDirection);
+  void collectStyleThemeReference(unsigned level, const boost::optional<long> &lineColour, const boost::optional<long> &fillColour,
+                                  const boost::optional<long> &shadowColour, const boost::optional<long> &fontColour);
+
+  virtual void collectMetaData(const librevenge::RVNGPropertyList &metaData);
+
 
   // Field list
   void collectFieldList(unsigned id, unsigned level);
@@ -186,7 +180,7 @@ public:
 private:
   VSDContentCollector(const VSDContentCollector &);
   VSDContentCollector &operator=(const VSDContentCollector &);
-  libwpg::WPGPaintInterface *m_painter;
+  librevenge::RVNGDrawingInterface *m_painter;
 
   void applyXForm(double &x, double &y, const XForm &xform);
 
@@ -204,24 +198,24 @@ private:
 
   void _handleLevelChange(unsigned level);
 
-  void _handleForeignData(const WPXBinaryData &data);
+  void _handleForeignData(const librevenge::RVNGBinaryData &data);
 
-  void _lineProperties(const VSDLineStyle &style, WPXPropertyList &styleProps);
-  void _fillAndShadowProperties(const VSDFillStyle &style, WPXPropertyList &styleProps);
+  void _lineProperties(const VSDLineStyle &style, librevenge::RVNGPropertyList &styleProps);
+  void _fillAndShadowProperties(const VSDFillStyle &style, librevenge::RVNGPropertyList &styleProps);
 
   void _applyLinePattern();
   const char *_linePropertiesMarkerViewbox(unsigned marker);
   const char *_linePropertiesMarkerPath(unsigned marker);
   double _linePropertiesMarkerScale(unsigned marker);
 
-  void appendCharacters(WPXString &text, const std::vector<unsigned char> &characters, TextFormat format);
-  void appendCharacters(WPXString &text, const std::vector<unsigned char> &characters);
-  void _convertDataToString(WPXString &result, const WPXBinaryData &data, TextFormat format);
-  bool parseFormatId( const char *formatString, unsigned short &result );
-  void _appendField(WPXString &text);
+  void appendCharacters(librevenge::RVNGString &text, const std::vector<unsigned char> &characters, TextFormat format);
+  void appendCharacters(librevenge::RVNGString &text, const std::vector<unsigned char> &characters);
+  void _convertDataToString(librevenge::RVNGString &result, const librevenge::RVNGBinaryData &data, TextFormat format);
+  bool parseFormatId(const char *formatString, unsigned short &result);
+  void _appendField(librevenge::RVNGString &text);
 
   // NURBS processing functions
-  bool _isUniform(const std::vector<double> weights) const;
+  bool _isUniform(const std::vector<double> &weights) const;
   void _generatePolylineFromNURBS(unsigned degree, const std::vector<std::pair<double, double> > &controlPoints,
                                   const std::vector<double> &knotVector, const std::vector<double> &weights);
   void _generateBezierSegmentsFromNURBS(unsigned degree, const std::vector<std::pair<double, double> > &controlPoints,
@@ -229,6 +223,9 @@ private:
   void _outputCubicBezierSegment(const std::vector<std::pair<double, double> > &points);
   void _outputQuadraticBezierSegment(const std::vector<std::pair<double, double> > &points);
   void _outputLinearBezierSegment(const std::vector<std::pair<double, double> > &points);
+  void _appendVisibleAndPrintable(librevenge::RVNGPropertyList &propList);
+  void _bulletFromParaFormat(VSDBullet &bullet, const VSDParaStyle &paraStyle);
+  void _listLevelFromBullet(librevenge::RVNGPropertyList &propList, const VSDBullet &bullet);
 
   bool m_isPageStarted;
   double m_pageWidth;
@@ -243,12 +240,12 @@ private:
   XForm m_xform;
   XForm *m_txtxform;
   VSDMisc m_misc;
-  std::vector<WPXPropertyList> m_currentFillGeometry;
-  std::vector<WPXPropertyList> m_currentLineGeometry;
+  std::vector<librevenge::RVNGPropertyList> m_currentFillGeometry;
+  std::vector<librevenge::RVNGPropertyList> m_currentLineGeometry;
   std::map<unsigned, XForm> *m_groupXForms;
-  WPXBinaryData m_currentForeignData;
-  WPXBinaryData m_currentOLEData;
-  WPXPropertyList m_currentForeignProps;
+  librevenge::RVNGBinaryData m_currentForeignData;
+  librevenge::RVNGBinaryData m_currentOLEData;
+  librevenge::RVNGPropertyList m_currentForeignProps;
   unsigned m_currentShapeId;
   unsigned m_foreignType;
   unsigned m_foreignFormat;
@@ -275,9 +272,9 @@ private:
 
   std::map<unsigned, NURBSData> m_NURBSData;
   std::map<unsigned, PolylineData> m_polylineData;
-  WPXBinaryData m_textStream;
-  std::map<unsigned, WPXString> m_names, m_stencilNames;
-  std::vector<WPXString> m_fields;
+  librevenge::RVNGBinaryData m_textStream;
+  std::map<unsigned, librevenge::RVNGString> m_names, m_stencilNames;
+  std::vector<librevenge::RVNGString> m_fields;
   VSDFieldList m_stencilFields;
   unsigned m_fieldIndex;
   TextFormat m_textFormat;
@@ -288,6 +285,8 @@ private:
   VSDFillStyle m_fillStyle;
   VSDTextBlockStyle m_textBlockStyle;
 
+  VSDThemeReference m_themeReference;
+
   VSDCharStyle m_defaultCharStyle;
   VSDParaStyle m_defaultParaStyle;
 
@@ -305,6 +304,8 @@ private:
   VSDPage m_currentPage;
   VSDPages m_pages;
 
+  VSDLayerList m_layerList;
+
   std::vector<std::pair<double, double> > m_splineControlPoints;
   std::vector<double> m_splineKnotVector;
   double m_splineX, m_splineY;
@@ -313,6 +314,10 @@ private:
   unsigned m_splineLevel;
   unsigned m_currentShapeLevel;
   bool m_isBackgroundPage;
+
+  VSDLayerList m_currentLayerList;
+  std::vector<unsigned> m_currentLayerMem;
+  std::vector<VSDTabSet> m_tabSets;
 };
 
 } // namespace libvisio
